#ifndef CONFMANAGER_H
#define CONFMANAGER_H

#include <QObject>
#include <QMap>

class QFileSystemWatcher;
class ConfManager : public QObject
{
    Q_OBJECT
public:

    enum Resource {
        CPU,
        IO,
        Memory
    };
    Q_ENUM(Resource);

    enum ResourceUrgency {
        Low = 1,
        Medium = 2,
        High = 4,
    };
    Q_ENUM(ResourceUrgency);

    typedef QMap<ResourceUrgency, int> ResourceUrgencyThrehold;

    explicit ConfManager(QObject *parent = nullptr);

    QStringList features() const;
    QList<Resource> pressureResouces() const;
    QMap<ResourceUrgency, QString> pressureTriggers(Resource resource) const;
    ResourceUrgencyThrehold resourceThreshold(Resource resource) const;
    int detectEffectiveNumber(Resource resource) const;
    int resourceUrgencyIndex(ResourceUrgency urgency) const;

private Q_SLOTS:
    void readConf();

private:
    void updateFeatures(const QJsonObject &jsonObj);
    void updateResourceConfig(const QString &resource,
                              const QString &api,
                              const QJsonArray &thresholds,
                              const int &effectiveNumber);

private:
    QStringList m_features;
    QMap<Resource, ResourceUrgencyThrehold> m_pressureTriggers;
    QMap<Resource, ResourceUrgencyThrehold> m_resourceThresholds;
    QMap<Resource, int> m_detectEffectiveNumbers;

    QFileSystemWatcher *confFileWatcher;
    const char *kConfFilename = "/etc/kylin-process-manager/kylin-process-manager.json";
};

#endif // CONFMANAGER_H
