#include "processmanager.h"
#include <QThread>
#include "appmanagercgroupd.h"
#include "pressurewatcher.h"
#include "systemresourcewatcher.h"
#include "processreclaimer.h"
#include "confmanager.h"
#include "common.h"

ProcessManager::ProcessManager(QObject *parent)
    : QObject{parent},
      m_appMgrCgroupd(new AppManagerCgroupd()),
      m_processReclaimer(new ProcessReclaimer(this))
{
    startWatchers();
}

void ProcessManager::startWatchers()
{
    QThread *pressureWatcherThread = new QThread(this);
    QThread *memResourceWatcheThread = new QThread(this);
    PressureWatcher *pressureWatcher = new PressureWatcher;
    SystemResourceWatcher *resourceWatcher = new SystemResourceWatcher;
    startWatcherThread(pressureWatcher, pressureWatcherThread);
    startWatcherThread(resourceWatcher, memResourceWatcheThread);
}

QMap<QString, QVariant> ProcessManager::CreateProcessCGroup(const QString &appId, int pid )
{
    return m_appMgrCgroupd->CreateProcessCGroup(appId, pid);
}

QMap<QString, QVariant> ProcessManager::CreateProcessCGroup(const QString &appId, const QList<int> &pids)
{
    return m_appMgrCgroupd->CreateProcessCGroup(appId, pids);
}

QMap<QString, QVariant> ProcessManager::DeleteProcessCGroup(const QString &cgroupName)
{
    return m_appMgrCgroupd->DeleteProcessCGroup(cgroupName);
}

QMap<QString, QVariant> ProcessManager::SetProcessCGroupResourceLimit(const QString &cgroupName,
                                                                      const QString &attrName,
                                                                      int value)
{
    return m_appMgrCgroupd->SetProcessCGroupResourceLimit(cgroupName, attrName, value);
}

QMap<QString,QVariant> ProcessManager::CGroupNameWithPid(int pid)
{
    return m_appMgrCgroupd->CGroupNameWithPid(pid);
}

QMap<QString,QVariant> ProcessManager::ReclaimProcesses(const QList<int> &pids)
{
    return m_processReclaimer->reclaimProcesses(pids);
}

QMap<QString, QVariant> ProcessManager::ReclaimProcesses(const QString &cgroup)
{
    auto pids = m_appMgrCgroupd->pids(cgroup);
    return m_processReclaimer->reclaimProcesses(pids);
}

QMap<QString, QVariant> ProcessManager::PidsOfCGroup(const QString &cgroupName)
{
    return m_appMgrCgroupd->PidsOfCGroup(cgroupName);
}

QStringList ProcessManager::Features() const
{
    ConfManager &confManager = common::Singleton<ConfManager>::GetInstance();
    QStringList confFeatures = confManager.features();
    if (confFeatures.contains("reclaim") && !ProcessReclaimer::isAvailable()) {
        confFeatures.removeOne("reclaim");
    }
    return confFeatures;
}

void ProcessManager::startWatcherThread(ResourceWatcher *watcher, QThread *thread)
{
    if (!watcher || !thread) {
        return;
    }

    watcher->moveToThread(thread);

    connect(thread, &QThread::started, watcher, &ResourceWatcher::start);
    connect(thread, &QThread::finished, thread, &QThread::deleteLater);
    connect(thread, &QThread::finished, watcher, &ResourceWatcher::deleteLater);
    connect(watcher, &ResourceWatcher::finished, thread, &QThread::quit);

    connect(watcher, &ResourceWatcher::ResourceThresholdWarning, this, &ProcessManager::ResourceThresholdWarning);

    thread->start();
}
