#include "processreclaimer.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>

ProcessReclaimer::ProcessReclaimer(QObject *parent)
    : QObject{parent},
      m_isAvailable(false)
{
    if (access("/proc/1/reclaim",F_OK) != -1) {
        m_isAvailable = true;
    }
}

bool ProcessReclaimer::isAvailable()
{
    return access("/proc/1/reclaim", F_OK) != -1;
}

common::DBusResult ProcessReclaimer::reclaimProcesses(const QList<int> pids)
{
    auto result = common::initDbusResult();
    if (pids.isEmpty()) {
        result[common::kDbusErrMsg] = "The pids is empty!";
        result[common::kDbusResult] = false;
        return result;
    }
    const char *content = "all";
    for (auto const &pid : qAsConst(pids)) {
        QString api = QString("/proc/%1/reclaim").arg(pid);
        QByteArray ba = api.toLocal8Bit();
        int fd = open(ba.data(), O_RDWR | O_NONBLOCK);
        if (fd < 0) {
            result[common::kDbusErrMsg] = "Open api file " + api + " failed, " + strerror(errno);
            result[common::kDbusResult] = false;
            return result;
        }
        if (write(fd, content, strlen(content) + 1) < 0) {
            result[common::kDbusErrMsg] = "Write api file " + api + " failed, " + strerror(errno);
            result[common::kDbusResult] = false;
            return result;
        }
    }
    result[common::kDbusResult] = true;
    return result;
}
